@@ -15,7 +15,7 @@ use crate::node::{
     network_knowledge::{SectionAuthorityProvider, SectionKeyShare},
     XorName,
 };
-use crate::types::{Peer, UnnamedPeer};
+use crate::types::Peer;
 
 use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
@@ -33,7 +33,7 @@ pub(crate) enum Cmd {
     /// Handle `message` from `sender`.
     /// Holding the WireMsg that has been received from the network,
     HandleMsg {
-        sender: UnnamedPeer,
+        sender: Peer,
         wire_msg: WireMsg,
         #[debug(skip)]
         // original bytes to avoid reserializing for entropy checks
