@@ -6,7 +6,7 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-mod agreement;
-mod anti_entropy;
-mod services;
-mod system;
+mod handling;
+mod sending;
+
+pub(crate) use handling::handle_proposal;
