@@ -11,7 +11,7 @@ mod system;
 
 use self::log_ctx::LogCtx;
 use std::time::Duration;
-use sysinfo::AsU32;
+use sysinfo::PidExt;
 use sysinfo::{System, SystemExt};
 use system::Process;
 use tokio::time::MissedTickBehavior;
